package com.trade.admin.system.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.trade.admin.system.domain.model.LoginUser;
import com.trade.admin.system.security.utils.SecurityUtils;
import com.trade.admin.system.utils.PageUtils;
import com.trade.common.core.domain.ResponseResult;
import com.trade.common.core.page.PageDomain;
import com.trade.common.core.page.TableDataInfo;
import com.trade.common.core.page.TableSupport;
import com.trade.common.sql.SqlUtil;
import com.trade.common.utils.DateUtils;
import com.trade.common.utils.LoggerUtil;
import com.trade.common.utils.StringUtils;
import org.slf4j.Logger;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;

import java.beans.PropertyEditorSupport;
import java.util.Date;
import java.util.List;

/**
 * web层通用数据处理
 * 
 * @author wl
 */
public class BaseController {

    protected final Logger logger = LoggerUtil.logger(LoggerUtil.LogFileNameEnum.SYS_LOG);;

    /**
     * 将前台传递过来的日期格式的字符串，自动转化为Date类型
     */
    @InitBinder
    public void initBinder(WebDataBinder binder) {
        // Date 类型转换
        binder.registerCustomEditor(Date.class, new PropertyEditorSupport() {

            @Override
            public void setAsText(String text) {
                setValue(DateUtils.parseDate(text));
            }
        });
    }

    /**
     * 设置请求分页数据
     */
    protected void startPage() {
        PageUtils.startPage();
    }

    /**
     * 设置请求排序数据
     */
    protected void startOrderBy() {
        PageDomain pageDomain = TableSupport.buildPageRequest();
        if (StringUtils.isNotEmpty(pageDomain.getOrderBy())) {
            String orderBy = SqlUtil.escapeOrderBySql(pageDomain.getOrderBy());
            PageHelper.orderBy(orderBy);
        }
    }

    /**
     * 清理分页的线程变量
     */
    protected void clearPage() {
        PageUtils.clearPage();
    }

    /**
     * 响应请求分页数据
     */
    @SuppressWarnings({
            "rawtypes", "unchecked"
    })
    protected <T> TableDataInfo getDataTable(List<T> list) {
        TableDataInfo rspData = new TableDataInfo();
        rspData.setRows(list);
        rspData.setTotal(new PageInfo(list).getTotal());
        clearPage();
        return rspData;
    }

    /**
     * 返回成功
     */
    public <T> ResponseResult<T> success() {
        return ResponseResult.success();
    }

    /**
     * 返回失败消息
     */
    public <T> ResponseResult<T>  error() {
        return ResponseResult.error();
    }

    /**
     * 返回成功消息
     */
    public <T> ResponseResult<T> success(String message) {
        return ResponseResult.success(message);
    }

    /**
     * 返回成功消息
     */
    public <T> ResponseResult<T> success(T data) {
        return ResponseResult.success(data);
    }

    /**
     * 返回失败消息
     */
    public <T> ResponseResult<T> error(String message) {
        return ResponseResult.error(message);
    }

    /**
     * 返回警告消息
     */
    public <T> ResponseResult<T> warn(String message) {
        return ResponseResult.warn(message);
    }

    /**
     * 响应返回结果
     * 
     * @param rows 影响行数
     * @return 操作结果
     */
    protected <T> ResponseResult<T> toAjax(int rows) {
        return rows > 0 ? ResponseResult.success() : ResponseResult.error();
    }

    /**
     * 响应返回结果
     * 
     * @param result 结果
     * @return 操作结果
     */
    protected <T> ResponseResult<T> toAjax(boolean result) {
        return result ? success() : error();
    }

    /**
     * 页面跳转
     */
    public String redirect(String url) {
        return StringUtils.format("redirect:{}", url);
    }

    /**
     * 获取用户缓存信息
     */
    public LoginUser getLoginUser() {
        return SecurityUtils.getLoginUser();
    }

    /**
     * 获取登录用户id
     */
    public Long getUserId() {
        return getLoginUser().getUserId();
    }

    /**
     * 获取登录部门id
     */
    public Long getDeptId() {
        return getLoginUser().getDeptId();
    }

    /**
     * 获取登录用户名
     */
    public String getUsername() {
        return getLoginUser().getUsername();
    }
}
