package com.trade.service.thirdparty.xiaotu.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.trade.common.constant.CacheConstants;
import com.trade.common.redis.RedisCache;
import com.trade.common.utils.http.HttpClientUtils;
import com.trade.common.utils.http.HttpConfig;
import com.trade.common.utils.http.HttpHeader;
import com.trade.service.thirdparty.xiaotu.configuration.XiaotuConfiguration;
import com.trade.service.thirdparty.xiaotu.dto.*;
import com.trade.service.thirdparty.xiaotu.service.XiaotuRequestService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Service
public class XiaotuRequestServiceImpl implements XiaotuRequestService {

    private static final Logger logger = LoggerFactory.getLogger(XiaotuRequestServiceImpl.class);
    private static final String CONTENT_TYPE_JSON = "application/json";
    @Resource
    private RedisCache redisCache;
    @Resource
    private XiaotuConfiguration xiaotuConfiguration;

    private HttpHeader getCommonHeader() {
        return HttpHeader.custom().other("accessClientId", xiaotuConfiguration.getAccessClientId())
                .other("ftype", xiaotuConfiguration.getFtype())
                .other("gatewaAppKey", xiaotuConfiguration.getAccessClientId());
    }

    private HttpHeader getBusinessCommonHeader(String xtUid, String xtEid) {
        return HttpHeader.custom().other("accessClientId", xiaotuConfiguration.getAccessClientId())
                .other("ftype", xiaotuConfiguration.getFtype())
                .other("gatewaAppKey", xiaotuConfiguration.getAccessClientId())
                .other("accessToken", getAccessToken(xtUid, xtEid));
    }

    /**
     * 获取小图accessToken 和小图约定token的有效期为20分钟
     * 
     * @param xtUid 小图用户id
     * @param xtEid 小图企业id
     * @return 小图accessToken
     */
    private String getAccessToken(String xtUid, String xtEid) {
        String cacheKey = CacheConstants.XIAOTU_ACCESS_TOKEN + "_" + xtUid + "_" + xtEid;
        String accessToken = redisCache.getCacheObject(cacheKey);
        if (StrUtil.isEmpty(accessToken)) {
            String resp = HttpClientUtils
                    .post(HttpConfig.custom().headers(getCommonHeader().contentType(CONTENT_TYPE_JSON).build())
                            .json(JSON.toJSONString(new RequestTokenDto(xtEid, xtUid)))
                            .url(xiaotuConfiguration.getGetAccessTokenUrl()));
            CommonResponse commonResponse = JSON.parseObject(resp, CommonResponse.class);
            if (commonResponse != null && !commonResponse.isSuccess()) {
                logger.error("获取小图accessToken失败,resp={}", resp);
                throw new RuntimeException("获取小图accessToken失败");
            }
            JSONObject jsonObject = JSON.parseObject(resp).getJSONObject("body");
            accessToken = (String) jsonObject.get("ACCESS_TOKEN");
            if (StringUtils.isEmpty(accessToken)) {
                logger.error("获取小图accessToken为空,resp={}", resp);
                throw new RuntimeException("获取小图accessToken为空");
            }
            redisCache.setCacheObject(cacheKey, accessToken, 10, TimeUnit.MINUTES);
            return accessToken;
        }
        return accessToken;
    }

    @Override
    public RegisterUserResponse registerUser(RegisterUserRequest request) {
        String resp = HttpClientUtils
                .post(HttpConfig.custom().headers(getCommonHeader().contentType(CONTENT_TYPE_JSON).build())
                        .json(JSON.toJSONString(request)).url(xiaotuConfiguration.getRegisterUserUrl()));
        CommonResponse commonResponse = JSON.parseObject(resp, CommonResponse.class);
        if (commonResponse != null && !commonResponse.isSuccess()) {
            logger.error("小图:用户信息注册失败,req={},resp={}", request, resp);
            throw new RuntimeException("小图:用户信息注册失败");
        }
        return JSON.parseObject(JSON.parseObject(resp).getString("body"), RegisterUserResponse.class);
    }

    @Override
    public Boolean updateUser(UpdateUserRequest request) {
        String resp = HttpClientUtils
                .post(HttpConfig.custom().headers(getCommonHeader().contentType(CONTENT_TYPE_JSON).build())
                        .json(JSON.toJSONString(request)).url(xiaotuConfiguration.getUpdateUserUrl()));
        CommonResponse commonResponse = JSON.parseObject(resp, CommonResponse.class);
        return commonResponse.isSuccess();
    }

    @Override
    public RegisterEnterpriseResponse registerEnterprise(RegisterEnterpriseRequest request) {
        String resp = HttpClientUtils
                .post(HttpConfig.custom().headers(getCommonHeader().contentType(CONTENT_TYPE_JSON).build())
                        .json(JSON.toJSONString(request)).url(xiaotuConfiguration.getRegisterEnterpriseUrl()));
        CommonResponse commonResponse = JSON.parseObject(resp, CommonResponse.class);
        if (commonResponse != null && !commonResponse.isSuccess()) {
            logger.error("小图:企业信息注册失败,req={},resp={}", request, resp);
            throw new RuntimeException("小图:企业信息注册失败");
        }
        return JSON.parseObject(JSON.parseObject(resp).getString("body"), RegisterEnterpriseResponse.class);
    }

    @Override
    public CommodityResponse taxRefundRateInquiry(CommodityRequest commodityRequest,
            BusinessCommonRequest businessCommonRequest) {
        String request = JSON.toJSONString(commodityRequest);
        String resp = HttpClientUtils.post(HttpConfig.custom()
                .headers(getBusinessCommonHeader(businessCommonRequest.getXtUid(), businessCommonRequest.getXtEid())
                        .contentType(CONTENT_TYPE_JSON).build())
                .json(JSON.toJSONString(commodityRequest)).url(xiaotuConfiguration.getTsslcxUrl()));
        CommonResponse<CommodityResponse> commonResponse = JSON.parseObject(resp, CommonResponse.class);
        if (commonResponse != null && !commonResponse.isSuccess()) {
            logger.error("小图:退税税率查询接口,req={},resp={}", request, resp);
            throw new RuntimeException("小图:退税税率查询接口失败");
        }
        return commonResponse.getBody();
    }

    @Override
    public CommodityCustomsTariffRespose commodityCustomsTariffInquiry(String spid,
            BusinessCommonRequest businessCommonRequest) {
        String resp = HttpClientUtils.post(HttpConfig.custom()
                .headers(getBusinessCommonHeader(businessCommonRequest.getXtUid(), businessCommonRequest.getXtEid())
                        .contentType(CONTENT_TYPE_JSON).build())
                .json(JSON.toJSONString(spid)).url(xiaotuConfiguration.getHgszcxUrl()));
        CommonResponse<CommodityCustomsTariffRespose> commonResponse = JSON.parseObject(resp, CommonResponse.class);
        if (commonResponse != null && !commonResponse.isSuccess()) {
            logger.error("小图:海关税则查询接口,req={},resp={}", spid, resp);
            throw new RuntimeException("小图:海关税则查询接口失败");
        }
        return commonResponse.getBody();
    }

    /**
     * 小图通用查询接口 - 带分页
     *
     * @param request 请求参数
     * @param <T> 返回类型
     * @return List<T>
     */
    public <T extends CommonResultRows> CommonQueryResponse<T> queryCommonResponse(CommonQueryRequest request) {
        String resp = HttpClientUtils
                .post(HttpConfig.custom().headers(getCommonHeader().contentType(CONTENT_TYPE_JSON).build())
                        .json(JSON.toJSONString(request)).url(request.getUrl()));
        CommonResponse<?> commonResponse = JSON.parseObject(resp, CommonResponse.class);
        logger.info("查询小图数据,req={},resp={}", request, resp);
        if (commonResponse == null || !commonResponse.isSuccess()) {
            throw new RuntimeException("查询小图数据失败");
        }
        return JSON.parseObject((String) commonResponse.getBody(), new TypeReference<CommonQueryResponse<T>>() {
        });
    }

    /**
     * 小图通用查询接口 - 仅返回rows
     * 
     * @param request 请求参数
     * @param <T> 返回类型
     * @return List<T>
     */
    public <T extends CommonResultRows> List<T> queryCommonResponseRows(CommonQueryRequest request) {
        CommonQueryResponse<T> commonQueryResponse = queryCommonResponse(request);
        return commonQueryResponse.getRows();
    }

    @Override
    public List<CustomsDeclarationResponse> customsDeclarationInquiry(CustomsDeclarationRequest request) {
        request.setUrl(xiaotuConfiguration.getCustomsDeclarationUrl());
        return queryCommonResponseRows(request);
    }
}
